package maps;



import java.util.Map;
import java.util.Set;
import java.util.HashMap;
import java.util.HashSet;
import java.util.TreeMap;
import java.util.TreeSet;

//Demonstration of the use of JCF Maps: HashMap and TreeMap
public class MapDemoApp {

	/**
	 * @param args not used
	 */
	public static void main(String[] args) {
//		Create a JCF Map with some number of initial available slots and load factor
//		A Map is a type of Set; that is, it cannot have duplicate KEYS		
		Map<String, Student> students 
		 = new HashMap<String, Student>(); 
		
		Set<Student> roster 
		 = new HashSet<Student>(); 

		addStudentsToMap( students ); // helper method that populates the Map
		addStudentsToSet( roster ); // populates the Set


		// Iterate the Keys in the Map
		System.out.println("\n***Set of unique Keys in the Map");

		// Set is like Collection, but Set doesn't have duplicates, and a Map can't have duplicate Keys
		Set<String> keySet = students.keySet();  
		for (String studentKey : keySet )
			System.out.println(studentKey);

		// iterate the Collection of Values in the Map; a Map MAY have duplicate VALUES
//		System.out.println("\n***Collection of non-unique Values in the Map");
//		Collection<Student> studentSet = students.values();
//		for (Student temp : studentSet )
//			System.out.println (temp);

		// iterate the Entry objects in the map
//		System.out.println("\n***Collection of Entries in the Map");
//		for (Map.Entry<String, Student> entry : students.entrySet()) {
//			System.out.println ( entry );
//		}

		// Iterate the Students in the Set
		System.out.println("\n***Set of unique Students in the Set");
		for( Student s: roster )
			System.out.println(s);
		
	}

	// helper class that populates a Map
	public static void addStudentsToMap( Map<String, Student> students) {
		String firstnames[] = {"Anne",  "Alice", "Alice",  "Candice", "Doctor", "Frank",  "Red"};
		String lastnames[] =  {"Arbor", "Arbor", "Arbor",  "Cane",    "Denton", "Furter", "Green"};
		String keys[] = 	  {"Arbor", "Arbor", "Arbor2", "Cane",    "Denton", "Furter", "Green"};

		for( int i=0; i<firstnames.length; i++ ) {
			// Student( lastName, firstName, key, mm-dd-yyyy, gpa, major_code, program_code)
			Student s = new Student(lastnames[i], firstnames[i], 100000+i, 4.0F, 1, 1);
			Student previous = students.put( keys[i], s); 
			if( previous != null )
				System.out.println("****Collision adding " + s + " to the Map; replacing " + previous);
		}
	}
	
	// helper class that populates a Set
	public static void addStudentsToSet( Set<Student> students) {
		String firstnames[] = {"Anne",  "Alice", "Alice",  "Candice", "Doctor", "Frank",  "Red"};
		String lastnames[] =  {"Arbor", "Arbor", "Arbor",  "Cane",    "Denton", "Furter", "Green"};

		for( int i=0; i<firstnames.length; i++ ) {
			// Student( lastName, firstName, key, mm-dd-yyyy, gpa, major_code, program_code)
			Student s = new Student(lastnames[i], firstnames[i], 100000+i, 4.0F, 1, 1);
			boolean wasAdded = students.add(s); 
			if( !wasAdded )
				System.out.println("****Collision adding " + s + " to the Set; not added.");
		}
	}
}
